function z = zernfun2A(p,r,theta,order,nflag)
% ZERNFUN2A Single-index Zernike functions on the unit circle.
%   Modified by DFS for Matlab 6.1, with different and selectable ordering
%
%   Z = ZERNFUN2A(P,R,THETA) returns the Pth Zernike functions evaluated
%   at positions (R,THETA) on the unit circle.  P is a vector of positive
%   integers, R is a vector of numbers between 0 and 1,
%   and THETA is a vector of angles.  R and THETA must have the same
%   length.  The output Z is a matrix with one column for every P-value,
%   and one row for every (R,THETA) pair.
%
%   Z = ZERNFUN2A(P,R,THETA,order) returns the Pth Zernike function in the
%   specified order.  No or empty 4th parameter, or 'fringe' returns the
%   U. of Arizona or "fringe" Zernike set.  In this case, P is limited to 
%   the range 0:36.  Specifying 'standard' or 'noll' provides output in
%   a different order.  In that case, the maximum value in P is not limited, but
%   the smallest value allowed is 1.  The original order specified by
%   Paul Fricker is produced when order is 'original'.
%
%   As the default, the 'fringe' and 'original' sets are not normalized
%   while the 'standard' set is normalized.
%
%   Z = ZERNFUN2A(P,R,THETA,order,'norm') allows one to override the
%   default normalization. Replace 'norm' with an empty matrix to disable normalization.
%   Normalization is defined such that the integral of (r * [Zp(r,theta)]^2)
%   over the unit circle (from r=0 to r=1, and theta=0 to theta=2*pi)
%   is unity.  For the non-normalized polynomials, max(Zp(r=1,theta))=1
%   for all p.
%
%
%   See also ZERNPOL, ZERNFUN.

%   Original: Paul Fricker 11/13/2006
%   Modification:  David F. Schaack 10/2/2009

if(nargin > 3)
    if(isempty(order) | strcmpi(order,'fringe'))
        Ordering = 'UofA';
        Normalized = 0;
    elseif(strcmpi(order,'standard') | strcmpi(order,'noll'))
        Ordering = 'Noll';
        Normalized = 1;
    elseif(strcmpi(order,'original'))
        Ordering = 'Orig';
        Normalized = 0;
    else
        error('Unknown ordering specified');
    end
else
    Ordering = 'UofA';
    Normalized = 0;
end

% Override default normalization if so specified
if(nargin > 4)
    if(strcmpi(nflag,'norm'))
        Normalized = 1;
    else
        Normalized = 0;
    end
end

% Check and prepare the inputs:
% -----------------------------
if min(size(p))~=1
    error('Input P must be vector.')
end

if (Ordering == 'UofA' & any(p>36))
    error('Fringe or UofA ordering specifies only the first 37 Zernike functions');
elseif(Ordering == 'Noll' & any(p<1))
    error('Standard or Noll ordering starts with Z1, not Z0');
elseif(Ordering == 'Orig' & any(p>35))
    error('Original ordering specified by Paul Fricker is restricted to the first 36 Zernike functions')
end

% Get the order and frequency corresonding to the function number:
% ----------------------------------------------------------------
p = p(:);
if Ordering == 'Noll'
    % Minimum value of p is 1 for this ordering
    % This ordering was defined in R. J. Noll, JOSA vol 66, pp. 207-211, 1976.
    % and is completely explained and documented by
    % V. N. Mahajan, Chapter 13, Optical Shop Testing, 3rd. Ed., D. Malacara. ed., Wiley,2007.
    n = fix(sqrt(2*p-1)+0.5)-1;
    m = zeros(size(n));
    ind = find(odd(n));
    pind = p(ind); nind = n(ind);
    m(ind) = 2*fix((2*(pind+1) - nind.*(nind+1))/4)-1;
    ind = find(even(n));
    pind = p(ind); nind = n(ind);
    m(ind) = 2*fix((2*pind+1 - nind.*(nind+1))/4);
	ind = find(even(p));
	m(ind) = -m(ind);
elseif Ordering == 'Orig'
	n = ceil((-3+sqrt(9+8*p))/2);
	m = 2*p - n.*(n+2);
elseif Ordering == 'UofA'
    % References:
    % <http://www.optics.arizona.edu/jcwyant/Zernikes/Zernikes.pdf>
    % <http://www.optics.arizona.edu/jcwyant/Zernikes/ZernikePolynomialsForTheWeb.pdf>
    % SYNOPSYS optical design code, Optical System Design, Inc., <www.osdoptics.com>
    % ZEMAX optical design code, Zemax Development Corp., <www.zemax.com>
    %
    % The "Fringe" or U of Arizona ordering is commonly used with polynomial numbers that begin both with 0 and with 1.
    % We specify that we begin with zero.  Also, some users put the piston term at the end.  We leave it at the beginning.
    % The UofA people defined n differently than others.  For them, n is not the highest power of r
    % in the Zernike polynomial; it is the order of the aberration in some other sense.
    % Therefore, rather than deal with the math necessary to translate, we simply tabulate:
    
    nbase = [0 1 1 2 2 2 3 3 4 3 3 4 4 5 5 6 4 4 5 5 6 6 7 7 8 5 5 6 6 7 7 8 8 9 9 10 12]';
    mbase = [0 -1 1 0 -2 2 -1 1 0 -3 3 -2 2 -1 1 0 -4 4 -3 3 -2 2 -1 1 0 -5 5 -4 4 -3 3 -2 2 -1 1 0 0]';
    pind = p+1; % Switch to MatLab base 1
    n = nbase(pind);
    m = mbase(pind); 
end

% Pass the inputs to the function ZERNFUN:
% ----------------------------------------
if(~Normalized)
    z = zernfun(n,m,r,theta);
else
    z = zernfun(n,m,r,theta,'norm');
end

% EOF zernfun2A